@@ -32,6 +32,7 @@
 #include "configmake.h"
 #include "remote/remote_protocol.h"
 #include "remote/remote_driver.h"
+#include "util/virnetdevopenvswitch.h"
 #include "virstring.h"
 #include "virutil.h"
 
@@ -152,7 +153,6 @@ daemonConfigNew(bool privileged ATTRIBUTE_UNUSED)
 
     data->prio_workers = 5;
 
-    data->max_requests = 20;
     data->max_client_requests = 5;
 
     data->audit_level = 1;
@@ -170,6 +170,8 @@ daemonConfigNew(bool privileged ATTRIBUTE_UNUSED)
     data->admin_keepalive_interval = 5;
     data->admin_keepalive_count = 5;
 
+    data->ovs_timeout = VIR_NETDEV_OVS_DEFAULT_TIMEOUT;
+
     localhost = virGetHostname();
     if (localhost == NULL) {
         /* we couldn't resolve the hostname; assume that we are
@@ -345,8 +347,6 @@ daemonConfigLoadOptions(struct daemonConfig *data,
     if (virConfGetValueUInt(conf, "prio_workers", &data->prio_workers) < 0)
         goto error;
 
-    if (virConfGetValueUInt(conf, "max_requests", &data->max_requests) < 0)
-        goto error;
     if (virConfGetValueUInt(conf, "max_client_requests", &data->max_client_requests) < 0)
         goto error;
 
@@ -388,6 +388,9 @@ daemonConfigLoadOptions(struct daemonConfig *data,
     if (virConfGetValueUInt(conf, "admin_keepalive_count", &data->admin_keepalive_count) < 0)
         goto error;
 
+    if (virConfGetValueUInt(conf, "ovs_timeout", &data->ovs_timeout) < 0)
+        goto error;
+
     return 0;
 
  error:
@@ -427,7 +430,7 @@ int daemonConfigLoadData(struct daemonConfig *data,
     virConfPtr conf;
     int ret;
 
-    conf = virConfReadMem(filedata, strlen(filedata), 0);
+    conf = virConfReadString(filedata, 0);
     if (!conf)
         return -1;
 
