@@ -71,7 +71,6 @@ struct daemonConfig {
 
     unsigned int prio_workers;
 
-    unsigned int max_requests;
     unsigned int max_client_requests;
 
     unsigned int log_level;
@@ -92,6 +91,8 @@ struct daemonConfig {
 
     int admin_keepalive_interval;
     unsigned int admin_keepalive_count;
+
+    unsigned int ovs_timeout;
 };
 
 
