@@ -29,6 +29,7 @@
 #include "virlog.h"
 #include "virnetserverclient.h"
 #include "virerror.h"
+#include "libvirt_internal.h"
 
 #define VIR_FROM_THIS VIR_FROM_STREAMS
 
@@ -52,6 +53,9 @@ struct daemonClientStream {
     virNetMessagePtr rx;
     bool tx;
 
+    bool allowSkip;
+    size_t dataLen; /* How much data is there remaining until we see a hole */
+
     daemonClientStreamPtr next;
 };
 
@@ -227,17 +231,23 @@ daemonStreamEvent(virStreamPtr st, int events, void *opaque)
         int ret;
         virNetMessagePtr msg;
         virNetMessageError rerr;
+        virErrorPtr origErr = virSaveLastError();
 
         memset(&rerr, 0, sizeof(rerr));
         stream->closed = true;
         virStreamEventRemoveCallback(stream->st);
         virStreamAbort(stream->st);
-        if (events & VIR_STREAM_EVENT_HANGUP)
-            virReportError(VIR_ERR_RPC,
-                           "%s", _("stream had unexpected termination"));
-        else
-            virReportError(VIR_ERR_RPC,
-                           "%s", _("stream had I/O failure"));
+        if (origErr && origErr->code != VIR_ERR_OK) {
+            virSetError(origErr);
+        } else {
+            if (events & VIR_STREAM_EVENT_HANGUP)
+                virReportError(VIR_ERR_RPC,
+                               "%s", _("stream had unexpected termination"));
+            else
+                virReportError(VIR_ERR_RPC,
+                               "%s", _("stream had I/O failure"));
+        }
+        virFreeError(origErr);
 
         msg = virNetMessageNew(false);
         if (!msg) {
@@ -285,7 +295,8 @@ daemonStreamFilter(virNetServerClientPtr client ATTRIBUTE_UNUSED,
 
     virMutexLock(&stream->priv->lock);
 
-    if (msg->header.type != VIR_NET_STREAM)
+    if (msg->header.type != VIR_NET_STREAM &&
+        msg->header.type != VIR_NET_STREAM_HOLE)
         goto cleanup;
 
     if (!virNetServerProgramMatches(stream->prog, msg))
@@ -321,7 +332,8 @@ daemonClientStream *
 daemonCreateClientStream(virNetServerClientPtr client,
                          virStreamPtr st,
                          virNetServerProgramPtr prog,
-                         virNetMessageHeaderPtr header)
+                         virNetMessageHeaderPtr header,
+                         bool allowSkip)
 {
     daemonClientStream *stream;
     daemonClientPrivatePtr priv = virNetServerClientGetPrivateData(client);
@@ -339,6 +351,7 @@ daemonCreateClientStream(virNetServerClientPtr client,
     stream->serial = header->serial;
     stream->filterID = -1;
     stream->st = st;
+    stream->allowSkip = allowSkip;
 
     return stream;
 }
@@ -648,6 +661,52 @@ daemonStreamHandleAbort(virNetServerClientPtr client,
 }
 
 
+static int
+daemonStreamHandleHole(virNetServerClientPtr client,
+                       daemonClientStream *stream,
+                       virNetMessagePtr msg)
+{
+    int ret;
+    virNetStreamHole data;
+
+    VIR_DEBUG("client=%p, stream=%p, proc=%d, serial=%u",
+              client, stream, msg->header.proc, msg->header.serial);
+
+    /* Let's check if client plays nicely and advertised usage of
+     * sparse stream upfront. */
+    if (!stream->allowSkip) {
+        virReportError(VIR_ERR_RPC, "%s",
+                       _("Unexpected stream hole"));
+        return -1;
+    }
+
+    if (virNetMessageDecodePayload(msg,
+                                   (xdrproc_t) xdr_virNetStreamHole,
+                                   &data) < 0)
+        return -1;
+
+    ret = virStreamSendHole(stream->st, data.length, data.flags);
+
+    if (ret < 0) {
+        virNetMessageError rerr;
+
+        memset(&rerr, 0, sizeof(rerr));
+
+        VIR_INFO("Stream send hole failed");
+        stream->closed = true;
+        virStreamEventRemoveCallback(stream->st);
+        virStreamAbort(stream->st);
+
+        return virNetServerProgramSendReplyError(stream->prog,
+                                                 client,
+                                                 msg,
+                                                 &rerr,
+                                                 &msg->header);
+    }
+
+    return 0;
+}
+
 
 /*
  * Called when the stream is signalled has being able to accept
@@ -666,19 +725,31 @@ daemonStreamHandleWrite(virNetServerClientPtr client,
         virNetMessagePtr msg = stream->rx;
         int ret;
 
-        switch (msg->header.status) {
-        case VIR_NET_OK:
-            ret = daemonStreamHandleFinish(client, stream, msg);
-            break;
+        if (msg->header.type == VIR_NET_STREAM_HOLE) {
+            /* Handle special case when the client sent us a hole.
+             * Otherwise just carry on with processing stream
+             * data. */
+            ret = daemonStreamHandleHole(client, stream, msg);
+        } else if (msg->header.type == VIR_NET_STREAM) {
+            switch (msg->header.status) {
+            case VIR_NET_OK:
+                ret = daemonStreamHandleFinish(client, stream, msg);
+                break;
 
-        case VIR_NET_CONTINUE:
-            ret = daemonStreamHandleWriteData(client, stream, msg);
-            break;
+            case VIR_NET_CONTINUE:
+                ret = daemonStreamHandleWriteData(client, stream, msg);
+                break;
 
-        case VIR_NET_ERROR:
-        default:
-            ret = daemonStreamHandleAbort(client, stream, msg);
-            break;
+            case VIR_NET_ERROR:
+            default:
+                ret = daemonStreamHandleAbort(client, stream, msg);
+                break;
+            }
+        } else {
+            virReportError(VIR_ERR_RPC,
+                           _("Unexpected message type: %d"),
+                           msg->header.type);
+            ret = -1;
         }
 
         if (ret > 0)
@@ -733,6 +804,8 @@ daemonStreamHandleRead(virNetServerClientPtr client,
     size_t bufferLen = VIR_NET_MESSAGE_LEGACY_PAYLOAD_MAX;
     int ret = -1;
     int rv;
+    int inData = 0;
+    long long length = 0;
 
     VIR_DEBUG("client=%p, stream=%p tx=%d closed=%d",
               client, stream, stream->tx, stream->closed);
@@ -757,6 +830,58 @@ daemonStreamHandleRead(virNetServerClientPtr client,
     if (!(msg = virNetMessageNew(false)))
         goto cleanup;
 
+    if (stream->allowSkip && stream->dataLen == 0) {
+        /* Handle skip. We want to send some data to the client. But we might
+         * be in a hole. Seek to next data. But if we are in data already, just
+         * carry on. */
+
+        rv = virStreamInData(stream->st, &inData, &length);
+        VIR_DEBUG("rv=%d inData=%d length=%lld", rv, inData, length);
+
+        if (rv < 0) {
+            if (virNetServerProgramSendStreamError(remoteProgram,
+                                                   client,
+                                                   msg,
+                                                   &rerr,
+                                                   stream->procedure,
+                                                   stream->serial) < 0)
+                goto cleanup;
+            msg = NULL;
+
+            /* We're done with this call */
+            goto done;
+        } else {
+            if (!inData && length) {
+                stream->tx = false;
+                msg->cb = daemonStreamMessageFinished;
+                msg->opaque = stream;
+                stream->refs++;
+                if (virNetServerProgramSendStreamHole(remoteProgram,
+                                                      client,
+                                                      msg,
+                                                      stream->procedure,
+                                                      stream->serial,
+                                                      length,
+                                                      0) < 0)
+                    goto cleanup;
+
+                msg = NULL;
+
+                /* We have successfully sent stream skip to the other side.
+                 * To keep streams in sync seek locally too. */
+                virStreamSendHole(stream->st, length, 0);
+                /* We're done with this call */
+                goto done;
+            }
+        }
+
+        stream->dataLen = length;
+    }
+
+    if (stream->allowSkip &&
+        bufferLen > stream->dataLen)
+        bufferLen = stream->dataLen;
+
     rv = virStreamRecv(stream->st, buffer, bufferLen);
     if (rv == -2) {
         /* Should never get this, since we're only called when we know
@@ -771,6 +896,9 @@ daemonStreamHandleRead(virNetServerClientPtr client,
             goto cleanup;
         msg = NULL;
     } else {
+        if (stream->allowSkip)
+            stream->dataLen -= rv;
+
         stream->tx = false;
         if (rv == 0)
             stream->recvEOF = true;
@@ -788,6 +916,7 @@ daemonStreamHandleRead(virNetServerClientPtr client,
         msg = NULL;
     }
 
+ done:
     ret = 0;
  cleanup:
     VIR_FREE(buffer);
